@@ -397,7 +397,7 @@ static int snd_ctl_shm_read(snd_ctl_t *ctl, snd_ctl_event_t *event)
 	return err;
 }
 
-snd_ctl_ops_t snd_ctl_shm_ops = {
+static const snd_ctl_ops_t snd_ctl_shm_ops = {
 	.close = snd_ctl_shm_close,
 	.nonblock = snd_ctl_shm_nonblock,
 	.async = snd_ctl_shm_async,
@@ -555,8 +555,7 @@ int snd_ctl_shm_open(snd_ctl_t **handlep, const char *name, const char *sockname
 	close(sock);
 	if (ctrl)
 		shmdt(ctrl);
-	if (shm)
-		free(shm);
+	free(shm);
 	return result;
 }
 
